from spacy.symbols import (
    ADJ,
    ADP,
    ADV,
    AUX,
    CCONJ,
    DET,
    INTJ,
    NOUN,
    NUM,
    PART,
    PRON,
    PROPN,
    PUNCT,
    SCONJ,
    SYM,
    VERB,
    X,
)

TAG_MAP = {
    "NOUN": {"pos": NOUN},
    "VERB": {"pos": VERB},
    "AUX": {"pos": AUX},
    "ADJ": {"pos": ADJ},
    "ADV": {"pos": ADV},
    "PRON": {"pos": PRON},
    "DET": {"pos": DET},
    "ADP": {"pos": ADP},
    "SCONJ": {"pos": SCONJ},
    "CCONJ": {"pos": CCONJ},
    "PART": {"pos": PART},
    "INTJ": {"pos": INTJ},
    "NUM": {"pos": NUM},
    "PROPN": {"pos": PROPN},
    "PUNCT": {"pos": PUNCT},
    "SYM": {"pos": SYM},
    "X": {"pos": X},
}



from ...attrs import LIKE_NUM, NORM

# Cardinal numbers in Creole
_num_words = set(
    """
zewo youn en de twa kat senk sis sèt uit nèf dis
onz douz trèz katoz kenz sèz disèt dizwit diznèf
vent trant karant sinkant swasant swasann-dis
san mil milyon milya
""".split()
)

# Ordinal numbers in Creole (some are French-influenced, some simplified)
_ordinal_words = set(
    """
premye dezyèm twazyèm katryèm senkyèm sizyèm sètvyèm uitvyèm nèvyèm dizyèm
onzèm douzyèm trèzyèm katozyèm kenzèm sèzyèm disetyèm dizwityèm diznèvyèm
ventyèm trantyèm karantyèm sinkantyèm swasantyèm
swasann-disyèm santyèm milyèm milyonnyèm milyadyèm
""".split()
)

NORM_MAP = {
    "'m": "mwen",
    "'w": "ou",
    "'l": "li",
    "'n": "nou",
    "'y": "yo",
    "’m": "mwen",
    "’w": "ou",
    "’l": "li",
    "’n": "nou",
    "’y": "yo",
    "m": "mwen",
    "n": "nou",
    "l": "li",
    "y": "yo",
    "w": "ou",
    "t": "te",
    "k": "ki",
    "p": "pa",
    "M": "Mwen",
    "N": "Nou",
    "L": "Li",
    "Y": "Yo",
    "W": "Ou",
    "T": "Te",
    "K": "Ki",
    "P": "Pa",
}


def like_num(text):
    text = text.strip().lower()
    if text.startswith(("+", "-", "±", "~")):
        text = text[1:]
    text = text.replace(",", "").replace(".", "")
    if text.isdigit():
        return True
    if text.count("/") == 1:
        num, denom = text.split("/")
        if num.isdigit() and denom.isdigit():
            return True
    if text in _num_words:
        return True
    if text in _ordinal_words:
        return True
    # Handle things like "3yèm", "10yèm", "25yèm", etc.
    if text.endswith("yèm") and text[:-3].isdigit():
        return True
    return False


def norm_custom(text):
    return NORM_MAP.get(text, text.lower())


LEX_ATTRS = {
    LIKE_NUM: like_num,
    NORM: norm_custom,
}



from typing import Optional

from ...language import BaseDefaults, Language
from .stop_words import STOP_WORDS
from .tokenizer_exceptions import TOKENIZER_EXCEPTIONS


class ScottishDefaults(BaseDefaults):
    tokenizer_exceptions = TOKENIZER_EXCEPTIONS
    stop_words = STOP_WORDS


class Scottish(Language):
    lang = "gd"
    Defaults = ScottishDefaults


__all__ = ["Scottish"]
